# Biblioteca de Sistemas Lineares
# Aula de Mehtodos Numehricos 2021
#
# Prof. Breno Imbiriba

# Esta biblioteca requer o pacote 
# numpy


####################################
# Rotina para resolucao de sistema Triangular Superior.
def ts(A,B):
    """
       Resolve o sistema linear AX=B, onde A eh uma matriz triangular superior.
       Entrada:
       A - matrix triangular superior
       B - vetor coluna

       Saida:
       X - vetor coluna com a solucao
    """


    # Copia os argumentos para poder manipula-los
    # Isso eh necessário em Python quando os argumentos sao vetores e matrizes.
    a=A.copy()
    b=B.copy()

    # Obtem dimensionalidade de B
    n=B.size

    # x será do mesmo tamanho que B mas inicializado com zeros
    x=B.copy()
    x[:]=0

    # Em python, vetores e matrizes tem índices comecando a partir do 0.
    # ou seja, x(i) será x(0), x(1), ..., x(n-1)

    # A funcao "range(a,b)" gera uma sequencia de números de a ateh b-1 !
    # Para se criar uma sequencia 1,2,3,4 temos range(1,5)
    # A funcao "range(b,a,-1)" gera uma sequencia de b descendo ateh a+1 !
    # Para se criar uma sequencia 4,3,2,1 temos range(4,0,-1)

    x[n-1]=b[n-1]/a[n-1,n-1];
    for k in range(n-1,0,-1):
        soma=b[k-1]
        for j in range(k+1,n+1):
            soma=soma-a[k-1,j-1]*x[j-1]
        x[k-1]=soma/a[k-1,k-1]

    return x
####################################




####################################
# Eliminacao Gaussiana
def eg(A,B):
    """Usa o mehtodo da Eliminacao Gaussiana para 
    transformar o sistema linear AX=B em um sistema Triangular superior,
    retornando a nova matriz At, triangular superior, e o vetor Bt, 
    tal que AtX=Bt tem a mesma solucao."""

    # Necessário em Python quando trabalhando com matrizes e vetores
    a=A.copy()
    b=B.copy()

    # Obtem número de equacoes a partir do tamanho do vetor b
    n=B.size
   
    # Loop da primeira ateh a penúltima linha (de 1 ateh n-1)
    for klin in range(1,n-1+1):
        #######################################
        # Aplica Pivoteamento
        # Busca linha com maior pivot
        w=abs(a[klin-1,klin-1])
        rlin=0
        for jlin in range(klin,n+1):
           if abs(a[jlin-1,klin-1])>w:
              w=abs(a[jlin-1,klin-1])
              rlin=jlin
        # Terminado. Troca as linhas "k" e "r"
        if rlin != 0:
            #print('Trocando ',klin,rlin)
            for jlin in range(1,n+1):
               vj=a[klin-1,jlin-1]
               a[klin-1,jlin-1]=a[rlin-1,jlin-1]
               a[rlin-1,jlin-1]=vj
            bt=b[klin-1]
            b[klin-1]=b[rlin-1]
            b[rlin-1]=bt
        #########################################
        # Calcula novos valores de A e B
        for ilin in range(klin+1,n+1):
            m=a[ilin-1,klin-1]/a[klin-1,klin-1]
            b[ilin-1]=b[ilin-1]-m*b[klin-1]
            for jlin in range(klin+1,n+1):
               a[ilin-1,jlin-1]=a[ilin-1,jlin-1]-m*a[klin-1,jlin-1]
    # Zera o triangulo inferior (desnecessario) 
    for klin in range(1,n+1):
        for jcol in range(1,klin-1+1):
            a[klin-1,jcol-1]=0
    return a,b
####################################


